const webpack = require('webpack')
const showProcess = require('./showProcess')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')

const ReplacePlugin = require('../plugin/replacePlugin')

const assetsLoader = require('./assetsLoader')

//css

const babelConf = require('./babelConf')

const NODE_ENV = process.env.NODE_ENV
const API_ENV = process.env.API_ENV
const browser = process.browser
const ssr = process.ssr

const os = require('os')

const cpuCount = os.cpus().length ? os.cpus().length : 2

const HappyPack = require('happypack')

const styleLoaders = require('./styleLoaders')
const stylusConf = require('./stylusConf')

function plugins(program, ksr, isDev) {
  const generateStyleLoaders = styleLoaders(isDev)

  const filename = ksr.filename
  const chunkFilename = ksr.chunkFilename

  const miniCssExtractPlugin = new MiniCssExtractPlugin({
    filename: `${filename}.css`,
    chunkFilename: `${chunkFilename}.css`,
    ignoreOrder: true
  })

  const plugin = [
    showProcess(),
    miniCssExtractPlugin,
    new webpack.NoEmitOnErrorsPlugin(),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify(NODE_ENV),
      'process.env.API_ENV': JSON.stringify(API_ENV),
      'process.browser': JSON.stringify(browser),
      'process.ssr': JSON.stringify(ssr)
    }), //配置在 webpack 中直接使用的全局常量,
    // 去掉sourcemap url 上报后再后台还原
    new ReplacePlugin(true)
  ]

  const rmEmptyLine = {
    loader: 'string-replace-loader',
    options: {
      search: '\\s*?$',
      replace: '',
      flags: 'g'
    }
  }

  const happyPackPlugin = [
    new HappyPack({
      id: 'babel',
      threads: cpuCount,
      loaders: [
        {
          loader: 'babel-loader',
          options: babelConf(program)
        }
      ]
    }),
    new HappyPack({
      id: 'pug',
      threads: cpuCount,
      loaders: ['raw-loader', 'pug-plain-loader', rmEmptyLine]
    }),
    new HappyPack({
      id: 'vuepug',
      threads: cpuCount,
      loaders: ['pug-plain-loader', rmEmptyLine]
    })
  ]

  Array.prototype.push.apply(plugin, happyPackPlugin)

  return plugin
}

module.exports = plugins
