const chalk = require ('chalk')
const _ = require ('lodash')

module.exports = function reporter(middlewareOptions, options) {
  const { log, state, stats } = options

  if (state) {
    const displayStats = middlewareOptions.stats !== false

    if (displayStats) {
      let statsOptions = _.defaultsDeep (middlewareOptions.stats, {
        assets: true,
        errors: true,
        warnings: true,
        errorDetails: true,
        cached: false,
        cachedAssets: false,
        chunks: true,
        chunkModules: false,
        chunkGroups: false,
        chunkOrigins: true,
        timings: true,
        depth: true,
        modules: false,
        moduleAssets: false,
        moduleTrace: false,
        maxModules: 0,
        performance: true,
        publicPath: true,
        children: false,
        colors: true,
      })
      let logger = log.info
      if (stats.hasErrors ()) {
        logger = log.error
      } else if (stats.hasWarnings ()) {
        logger = log.warn
      }
      logger (stats.toString (statsOptions))
    }

    let message = chalk.green ('Compiled successfully.')
    if (stats.hasErrors ()) {
      message = chalk.red ('Failed to compile.')
    } else if (stats.hasWarnings ()) {
      message = chalk.yellow ('Compiled with warnings.')
    }
    log.info (message)
  } else {
    log.info (chalk.green ('Compiling...'))
  }
}
