const webpack = require("webpack");
const mock = require("./mock");
const chalk = require("chalk");
const devWebpackConf = require("../configuration/webpack/devWebpackConf");
const getConf = require("@ksyun/ksr-libs/libs/getConf");
const _ = require("lodash");
const koaWebpack = require("koa-webpack");
const reporter = require("../configuration/webpack/reporter");
const Vray = require("vray");
const app = new Vray();

module.exports = function(program) {
  const base = devWebpackConf(program, "dev");
  const baseWebpack = base.webpack;
  const baseKsr = base.ksr;
  const baseKsrServe = base.ksrServe;

  const ksrServe = getConf(program.serverConfig);

  const coverConfig = {};
  const config = _.defaults({}, coverConfig, program, baseKsrServe, ksrServe);

  const mergeProxy = _.defaults(
    {},
    getConf(baseKsrServe["proxy"]),
    getConf(ksrServe["proxy"]),
    getConf(program.proxy)
  );

  const port = config.port;
  const host = config.host;
  const mockfile = config.mock;
  const mhost = config.mhost;
  const mport = config.mport;
  const staticDir = config.static;
  const middleware = config.middleware;

  app.mountProxy(mergeProxy);
  app.mountDefaultMiddle();
  app.mountMiddle(middleware);
  app.mountStatic(staticDir);

  const compiler = webpack(baseWebpack);
  const hot = _.defaultsDeep(baseKsr.hot, {
    allEntries: true,
    reload: true,
    hmr: true,
    host: host === '0.0.0.0' ? {
      client: '*',
      server: host
    } : host ,
    autoConfigure: true
  });
  const dev = _.defaultsDeep(baseKsr.dev, {
    headers: {
      "Access-Control-Allow-Origin": "*",
      "Access-Control-Request-Method": "GET,POST,OPTIONS,DELETE,PUT",
      "Access-Control-Allow-Headers":
        "X-Requested-With,User-Agent,Keep-Alive,Content-Type",
      Pragma: "no-cache",
      ETag: "no-cache",
      "Cache-Control": "no-cache, no-store, must-revalidate",
      Expires: "Wed, 26 Feb 1997 08:21:57 GMT"
    },
    // contentBase: baseWebpack.output.path,
    publicPath: baseWebpack.output.publicPath,
    reporter: reporter,
    watchOptions: {
      aggregateTimeout: 500,
      poll: 1000,
      ignored: /node_modules/
    }
  });

  koaWebpack({
    compiler: compiler,
    devMiddleware: dev,
    hotClient: hot
  }).then(devMiddle => {
    app.use(devMiddle);
    app.listen(port, host, function(err) {
      if (err) {
        console.log(chalk.red(err));
        process.exit(1);
      }
      console.log(
        chalk.green(
          `Dev server listening at ${chalk.underline(`http://${host}:${port}`)}`
        )
      );
    });
    //mock serve
    const mockOpt = {
      files: mockfile,
      host: mhost,
      port: mport
    };

    mock(mockOpt);
  });
};
